@@ -28,13 +28,17 @@ namespace Glom
 {
 
 Dialog_Import_CSV_Progress::Dialog_Import_CSV_Progress(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
-: Gtk::Dialog(cobject), m_data_source(NULL), m_current_row(0)
+: Gtk::Dialog(cobject), 
+  m_data_source(0), 
+  m_current_row(0)
 {
   builder->get_widget("import_csv_progress_progress_bar", m_progress_bar);
   builder->get_widget("import_csv_progress_textview", m_text_view);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(!m_progress_bar || !m_text_view)
     throw std::runtime_error("Missing widgets from glade file for Dialog_Import_CSV_Progress");
+#endif    
 }
 
 bool Dialog_Import_CSV_Progress::init_db_details(const Glib::ustring& table_name)
@@ -53,16 +57,16 @@ void Dialog_Import_CSV_Progress::import(Dialog_Import_CSV& data_source)
   m_data_source = &data_source;
   m_current_row = 0;
 
-  switch(data_source.get_state())
+  switch(data_source.get_parser_state())
   {
-  case Dialog_Import_CSV::PARSING:
+  case CsvParser::PARSING:
     // Wait for the parsing to finish. We do not start importing before the file has been
     // parsed completely since we would not to rollback our changes in case of a
     // parsing error.
     m_progress_bar->set_text(Glib::ustring::compose(_("Parsing CSV file %1"), data_source.get_filename()));
     m_ready_connection = data_source.signal_state_changed().connect(sigc::mem_fun(*this, &Dialog_Import_CSV_Progress::on_state_changed));
     break;
-  case Dialog_Import_CSV::PARSED:
+  case CsvParser::PARSED:
     begin_import();
     break;
   default:
@@ -81,7 +85,7 @@ void Dialog_Import_CSV_Progress::clear()
   m_progress_connection.disconnect();
   m_ready_connection.disconnect();
 
-  m_data_source = NULL;
+  m_data_source = 0;
   m_current_row = 0;
 }
 
@@ -104,11 +108,11 @@ void Dialog_Import_CSV_Progress::on_state_changed()
 {
   switch(m_data_source->get_state())
   {
-  case Dialog_Import_CSV::ENCODING_ERROR:
+  case CsvParser::ENCODING_ERROR:
     // Cancel on error
     response(Gtk::RESPONSE_CANCEL);
     break;
-  case Dialog_Import_CSV::PARSED:
+  case CsvParser::PARSED:
     // Begin importing when fully parsed
     begin_import();
     //m_progress_connection = Glib::signal_idle().connect(sigc::mem_fun(*this, &Dialog_Import_CSV_Progress::on_idle_import));
@@ -144,11 +148,11 @@ bool Dialog_Import_CSV_Progress::on_idle_import()
     const sharedptr<Field>& field = m_data_source->get_field_for_column(i);
     if(field)
     {
-      // We always assume exported data is in postgres format, since
+      // We always assume exported data is in standard CSV format, since
       // we export it this way.
-      // TODO: Document what that format is.
+      const Glib::ustring str = m_data_source->get_data(m_current_row, i);
       bool success = false;
-      Gnome::Gda::Value value = field->from_file_format(m_data_source->get_data(m_current_row, i), success);
+      Gnome::Gda::Value value = field->from_file_format(str, success);
 
       if(success)
       {
@@ -257,7 +261,11 @@ void Dialog_Import_CSV_Progress::set_primary_key_value(const Gtk::TreeModel::ite
 
 Gnome::Gda::Value Dialog_Import_CSV_Progress::get_primary_key_value(const Gtk::TreeModel::iterator& /* row */) const
 {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   throw std::logic_error("Dialog_Import_CSV_Progress::get_primary_key_value() called");
+#endif
+
+  return Gnome::Gda::Value();
 }
 
 } //namespace Glom
