@@ -21,26 +21,40 @@
 #ifndef GLOM_DIALOG_PROGRESS_CREATING_H
 #define GLOM_DIALOG_PROGRESS_CREATING_H
 
-#include <libglademm.h>
+#include <gtkmm/builder.h>
 #include <gtkmm/window.h>
+#include <gtkmm/label.h>
 #include <gtkmm/progressbar.h>
 
 namespace Glom
 {
 
 /** Use this to show the user that something is happening.
+ * Call pulse() repeatedly to show that we are still working. 
  */
 class Dialog_ProgressCreating
   : public Gtk::Window
 {
 public:
-  Dialog_ProgressCreating(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Dialog_ProgressCreating(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_ProgressCreating();
 
+  void set_message(const Glib::ustring& title, const Glib::ustring& secondary_text);
+
   void pulse();
 
-protected:
+  // TODO: This should probably derive from Gtk::Dialog... armin.
+  void response(int response_id);
+  int run();
+
+  virtual bool on_delete_event(GdkEventAny* event);
+
+private:
   Gtk::ProgressBar* m_progress;
+  Gtk::Label* m_label_message;
+
+  int m_response_id;
+  bool m_running;
 };
 
 } //namespace Glom
