@@ -21,7 +21,10 @@
 #ifndef GLOM_MODE_DATA_CALCINPROGRESS_H
 #define GLOM_MODE_DATA_CALCINPROGRESS_H
 
-#include <glom/libglom/data_structure/field.h>
+#include <libglom/data_structure/field.h>
+
+namespace Glom
+{
 
 class CalcInProgress
 {
@@ -34,5 +37,7 @@ public:
   bool m_calc_finished;
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_CALCINPROGRESS_H
 
