@@ -24,11 +24,7 @@
 #include <libgdamm.h>
 #include <libglom/connectionpool_backends/backend.h>
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_CLIENT_ONLY
-
-#ifndef GLOM_ENABLE_POSTGRESQL
-# error The Glom <postgres.h> backend has been included even though PostgreSQL support is disabled
-#endif
+#include <libglom/libglom_config.h>
 
 namespace Glom
 {
@@ -54,19 +50,18 @@ public:
    *
    * @results True if everything is OK.
    */
-  static bool check_postgres_gda_client_is_available_with_warning();
+  static bool check_postgres_gda_client_is_available();
 
 private:
   virtual Field::sql_format get_sql_format() const { return Field::SQL_FORMAT_POSTGRES; }
   virtual bool supports_remote_access() const { return true; }
   virtual Glib::ustring get_string_find_operator() const { return "ILIKE"; } // ILIKE is a postgres extension for locale-dependent case-insensitive matches.
+  virtual const char* get_public_schema_name() const { return "public"; }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual bool change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& new_fields, std::auto_ptr<Glib::Error>& error);
+  virtual bool change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields, std::auto_ptr<Glib::Error>& error);
 
 protected:
   bool attempt_create_database(const Glib::ustring& database_name, const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<Glib::Error>& error);
-#endif
 
 protected:
   Glib::RefPtr<Gnome::Gda::Connection> attempt_connect(const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<ExceptionConnection>& error);
