@@ -19,8 +19,8 @@
  */
 
 #include "layoutitem_image.h"
-#include <glom/libglom/utils.h>
-#include <glom/libglom/data_structure/glomconversions.h>
+#include <libglom/utils.h>
+#include <libglom/data_structure/glomconversions.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -79,11 +79,6 @@ Gnome::Gda::Value LayoutItem_Image::get_image() const
   return m_image;
 }
 
-Glib::RefPtr<Gdk::Pixbuf> LayoutItem_Image::get_image_as_pixbuf() const
-{
-  return Conversions::get_pixbuf_for_gda_value(m_image);
-}
-
 void LayoutItem_Image::set_image(const Gnome::Gda::Value& image)
 {
   m_image = image;
