@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
  
-#include "layoutitem_fieldsummary.h"
+#include <libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -61,7 +61,7 @@ LayoutItem_FieldSummary& LayoutItem_FieldSummary::operator=(const LayoutItem_Fie
 
 Glib::ustring LayoutItem_FieldSummary::get_title_or_name() const
 {
-  Glib::ustring field_title = m_field->get_title_or_name();
+  const Glib::ustring field_title = get_full_field_details()->get_title_or_name();
 
   return get_summary_type_name(m_summary_type) + ": " + field_title; //TODO: Allow a more human-readable title for summary headings.
 }
