@@ -21,7 +21,7 @@
 #ifndef DOCUMENT_GLOM_H
 #define DOCUMENT_GLOM_H
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_CLIENT_ONLY, GLOM_ENABLE_SQLITE
+#include <libglom/libglom_config.h>
 
 #include <libglom/document/bakery/document_xml.h>
 #include <libglom/data_structure/relationship.h>
@@ -56,9 +56,7 @@ public:
   Document();
   virtual ~Document();
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual void set_modified(bool value = true);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   virtual void set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension = false);
 
@@ -66,7 +64,7 @@ public:
    * bool indicates success.
    * This is just here so the SWIG Java API generator does not need to wrap methods from the base classes.
    */
-  bool load();
+  bool load(int& failure_code);
   
   
   /** Whether the document was opened from another networked glom instance, 
@@ -75,13 +73,11 @@ public:
   void set_opened_from_browse(bool val = true);
   bool get_opened_from_browse() const;
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   /** The document usually saves itself when you call set_modified().
    * Pass false to this function to prevent that temporarily.
    * The document will be saved, if necessary, after you call this function with true.
    */
   void set_allow_autosave(bool value = true);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   bool get_is_example_file() const;
   void set_is_example_file(bool value = true);
@@ -100,21 +96,10 @@ public:
   /// How the database is hosted.
   enum HostingMode
   {
-    //Only enabled if GLOM_ENABLE_POSTGRESQL is defined:
     HOSTING_MODE_POSTGRES_CENTRAL, /*!< The database is hosted on an external postgresql server. */
-    
-    //Only enabled if GLOM_ENABLE_POSTGRESQL is defined and GLOM_ENABLE_CLIENT_ONLY is not defined:
     HOSTING_MODE_POSTGRES_SELF, /*!< A new postgres database process is spawned that hosts the data. */
-
-    //Only enabled if GLOM_ENABLE_SQLITE is defined:
     HOSTING_MODE_SQLITE, /*!< A sqlite database file is used. */
-
-    //This reduces the ifdefs elsewhere:
-#ifdef GLOM_ENABLE_POSTGRESQL
-    DEFAULT_HOSTED = HOSTING_MODE_POSTGRES_SELF
-#else
-    DEFAULT_HOSTED = HOSTING_MODE_SQLITE
-#endif
+    HOSTING_MODE_DEFAULT = HOSTING_MODE_POSTGRES_SELF /*!- Arbitrary default. */
   };
 
   /** Set the hosting mode of the database. 
@@ -138,11 +123,17 @@ public:
   bool get_network_shared() const;
 
   void set_connection_server(const Glib::ustring& strVal);
-  void set_connection_user(const Glib::ustring& strVal);
   void set_connection_database(const Glib::ustring& strVal);
   void set_connection_port(int port_number);
   void set_connection_try_other_ports(bool val);
 
+ /** Temporarily set a username in the document.
+   * Note that this is not saved in the document's file.
+   *
+   * TODO: Remove this, and just store it in ConnectionPool?
+   */
+  void set_connection_user(const Glib::ustring& strVal);
+
   /** If the database should be hosted, this provides the 
     * path to the directory that contains all the files needed to do that.
     * This is usually a specifically-named directory at the same level as the .glom file. 
@@ -152,11 +143,17 @@ public:
   std::string get_connection_self_hosted_directory_uri() const;
 
   Glib::ustring get_connection_server() const;
-  Glib::ustring get_connection_user() const;
   Glib::ustring get_connection_database() const;
   int get_connection_port() const;
   bool get_connection_try_other_ports() const;
 
+  /** Retrieve a username previously set in the document.
+   * Note that this is not saved in the document's file.
+   *
+   * TODO: Remove this, and just store it in ConnectionPool?
+   */
+  Glib::ustring get_connection_user() const;
+
   /** Set the language/locale used by original titles.
    * Title translations are translations of the text in this language.
    * @param locale: For instance, "en_US.UTF-8".
@@ -397,8 +394,6 @@ public:
   void emit_userlevel_changed();
 #endif //SWIG
 
-  static Glib::ustring get_default_layout_platform();
-
   /** This is transitory information, not saved to disk.
    */
   Glib::ustring get_active_layout_platform() const;
@@ -418,10 +413,8 @@ protected:
   //TODO: Remove this:
   friend class ReportBuilder;
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   ///If value is equal to the default then no attribute will be set, to save text space in the XML file.
   static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, guint value, guint value_default = 0);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
   
   // This is required by the report builder, so it cannot be disabled
   // in client only mode
@@ -436,10 +429,16 @@ public:
   static bool get_relationship_is_system_properties(const sharedptr<const Relationship>& relationship);
 #endif //SWIG
 
+  /// Failure codes that could be returned by load_after()
+  enum load_failure_codes
+  {
+    // The error codes must start at 1, so do that here:
+    LOAD_FAILURE_CODE_FILE_VERSION_TOO_NEW = 1
+  };
+
 private:
   //Overrides:
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual bool save_before();
   void save_before_layout_group(xmlpp::Element* node, const sharedptr<const LayoutGroup>& group, bool with_print_layout_positions = false);
   void save_before_sort_by(xmlpp::Element* node, const LayoutItem_GroupBy::type_list_sort_fields& list_fields);
@@ -451,9 +450,8 @@ private:
   void save_before_print_layout_position(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem>& item);
 
   void save_changes();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  virtual bool load_after();
+  virtual bool load_after(int& failure_code);
   void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring& table_name, const sharedptr<LayoutGroup>& group, bool with_print_layout_positions = false);
   void load_after_sort_by(const xmlpp::Element* node, const Glib::ustring& table_name, LayoutItem_GroupBy::type_list_sort_fields& list_fields);
   void load_after_layout_item_usesrelationship(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<UsesRelationship>& item);
@@ -474,12 +472,11 @@ private:
   static double get_node_attribute_value_as_decimal_double(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
   static float get_node_attribute_value_as_float(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true, bool value_default = false);
   static void set_node_attribute_value_as_float( xmlpp::Element* node, const Glib::ustring& strAttributeName, float value );
   static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
   static void set_node_text_child_as_value(xmlpp::Element* node, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
+
   static Gnome::Gda::Value get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type);
   static Gnome::Gda::Value get_node_text_child_as_value(const xmlpp::Element* node, Field::glom_field_type field_type);
 
@@ -492,7 +489,8 @@ private:
   HostingMode m_hosting_mode;
   bool m_network_shared;
 
-  Glib::ustring m_connection_server, m_connection_user, m_connection_database;
+  Glib::ustring m_connection_server, m_connection_database;
+  Glib::ustring m_connection_user; //Don't save the user.
   int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
   bool m_connection_try_other_ports; //Set to false for self-hosted or browsed-from-network documents.
 
@@ -596,9 +594,7 @@ private:
 
   bool m_block_cache_update; //For efficiency.
   bool m_block_modified_set;
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   bool m_allow_auto_save;
-#endif // !GLOM_ENABLE_CLIENT_ONLY
   bool m_is_example;
   guint m_document_format_version;
 
