@@ -39,9 +39,6 @@ public:
   type_signal_clicked signal_clicked();
   
 private:
-#ifndef GLIBMM_VFUNCS_ENABLED
-  static gboolean activate_impl(GtkCellRenderer* cell, GdkEvent* event, GtkWidget* widget, const gchar* path, GdkRectangle* background_area, GdkRectangle* cell_area, GtkCellRendererState flags);
-#endif
 
   virtual bool activate_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags);
 
