@@ -80,7 +80,7 @@ ImageGlom::~ImageGlom()
 void ImageGlom::set_layout_item(const sharedptr<LayoutItem>& layout_item, const Glib::ustring& table_name)
 {
   LayoutWidgetField::set_layout_item(layout_item, table_name);
-#ifndef GLOM_ENABLE_MAEMO  
+#ifdef GTKMM_ATKMM_ENABLED
   get_accessible()->set_name(layout_item->get_name());
 #endif  
 }
@@ -139,6 +139,8 @@ bool ImageGlom::on_button_press_event(GdkEventButton *event)
 
 #ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   return Gtk::EventBox::on_button_press_event(event);
+#else
+  return false;
 #endif // GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
 }
 
@@ -398,6 +400,7 @@ void ImageGlom::on_menupopup_activate_select_file()
   if(m_read_only)
     return;
 
+  //TODO: Use Hildon::FileChooser for Maemo.
   Gtk::FileChooserDialog dialog(_("Choose Image"), Gtk::FILE_CHOOSER_ACTION_OPEN);
 
   //Get image formats only:
